//===========================================================================
// GoTools - SINTEF Geometry Tools
//
// GoTools module: Intersections, version 1.0
//
// Copyright (C) 2000-2007 SINTEF ICT, Applied Mathematics, Norway.
//
// This program is free software; you can redistribute it and/or          
// modify it under the terms of the GNU General Public License            
// as published by the Free Software Foundation version 2 of the License. 
//
// This program is distributed in the hope that it will be useful,        
// but WITHOUT ANY WARRANTY; without even the implied warranty of         
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          
// GNU General Public License for more details.                           
//
// You should have received a copy of the GNU General Public License      
// along with this program; if not, write to the Free Software            
// Foundation, Inc.,                                                      
// 59 Temple Place - Suite 330,                                           
// Boston, MA  02111-1307, USA.                                           
//
// Contact information: E-mail: tor.dokken@sintef.no                      
// SINTEF ICT, Department of Applied Mathematics,                         
// P.O. Box 124 Blindern,                                                 
// 0314 Oslo, Norway.                                                     
//
// Other licenses are also available for this software, notably licenses
// for:
// - Building commercial software.                                        
// - Building software whose source code you wish to keep private.        
//===========================================================================
#ifndef _CVCVINTERSECTOR_H
#define _CVCVINTERSECTOR_H


#include "Intersector2Obj.h"


namespace Go {
///\addtogroup intersections
///\{



/// Class that performs intersection between two parametric curves.

class CvCvIntersector : public Intersector2Obj {
public:

    /// Constructor.
    /// The last two variables pertains only if the parent has one
    /// more parameter than the Intersector to be constructed.
    /// \param curve1 of type ParamCurveInt.
    /// \param curve2 of type ParamCurveInt.
    /// \param epsge the associated tolerance.
    /// \param prev the "parent" Intersector (0 if there is no
    /// parent).
    /// \param eliminated_parameter the index of the parameter that
    /// was removed from the parent \a prev.
    /// \param eliminated_value the value of the parameter that was
    /// removed from the parent \a prev.
    CvCvIntersector(boost::shared_ptr<ParamGeomInt> curve1, 
		    boost::shared_ptr<ParamGeomInt> curve2,
		    double epsge,
		    Intersector* prev = 0,
		    int eliminated_parameter = -1,
		    double eliminated_value = 0);

    /// Constructor.
    /// The last two variables pertains only if the parent has one
    /// more parameter than the Intersector to be constructed.
    /// \param curve1 of type ParamCurveInt.
    /// \param curve2 of type ParamCurveInt.
    /// \param epsge the associated tolerance.
    /// \param prev the "parent" Intersector (0 if there is no
    /// parent).
    /// \param eliminated_parameter the index of the parameter that
    /// was removed from the parent \a prev.
    /// \param eliminated_value the value of the parameter that was
    /// removed from the parent \a prev.
    CvCvIntersector(boost::shared_ptr<ParamGeomInt> curve1, 
		    boost::shared_ptr<ParamGeomInt> curve2,
		    boost::shared_ptr<GeoTol> epsge, 
		    Intersector* prev = 0,
		    int eliminated_parameter = -1,
		    double eliminated_value = 0);

    /// Destructor.
    virtual ~CvCvIntersector();

//     // Validation of given intersection results
//     virtual void validate(int level, ValidationStat status);

    /// Return the number of parameter directions for the
    /// intersection.
    /// \return The number of parameter directions for the
    /// intersection.
    virtual int numParams() const
    { return 2; }
	
protected:
    // Data members

    virtual boost::shared_ptr<Intersector> 
    lowerOrderIntersector(boost::shared_ptr<ParamGeomInt> obj1,
			  boost::shared_ptr<ParamGeomInt> obj2, 
			  Intersector* prev = 0,
			  int eliminated_parameter = -1,
			  double eliminated_value = 0);

    virtual int performRotatedBoxTest(double eps1, double eps2);

    virtual bool foundIntersectionNearBoundary();

    virtual int simpleCase2(Point& axis1, Point& axis2);

    virtual int checkCoincidence();

    virtual void microCase();
  
    virtual int updateIntersections();

    virtual int repairIntersections()
    { return 0; }

    virtual int linearCase();

    virtual int doSubdivide();

    virtual void writeOut();

private:
    void doIterate(double& par1, double& par2,
		   double& dist, double *guess=0);

    double distInCandidatePar(double par, int dir, const double* seed);

    int sortParameterDirections(int perm[]);

    SubdivisionClassification getSubdivisionParameter(int dir,
						      double& par);

};


///\}
} // namespace Go


#endif  // _CVCVINTERSECTOR_H

