//===========================================================================
// GoTools - SINTEF Geometry Tools
//
// GoTools module: Intersections, version 1.0
//
// Copyright (C) 2000-2007 SINTEF ICT, Applied Mathematics, Norway.
//
// This program is free software; you can redistribute it and/or          
// modify it under the terms of the GNU General Public License            
// as published by the Free Software Foundation version 2 of the License. 
//
// This program is distributed in the hope that it will be useful,        
// but WITHOUT ANY WARRANTY; without even the implied warranty of         
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          
// GNU General Public License for more details.                           
//
// You should have received a copy of the GNU General Public License      
// along with this program; if not, write to the Free Software            
// Foundation, Inc.,                                                      
// 59 Temple Place - Suite 330,                                           
// Boston, MA  02111-1307, USA.                                           
//
// Contact information: E-mail: tor.dokken@sintef.no                      
// SINTEF ICT, Department of Applied Mathematics,                         
// P.O. Box 124 Blindern,                                                 
// 0314 Oslo, Norway.                                                     
//
// Other licenses are also available for this software, notably licenses
// for:
// - Building commercial software.                                        
// - Building software whose source code you wish to keep private.        
//===========================================================================
#include "AlgObj2DInt.h"
#include "errormacros.h"


using std::vector;


namespace Go {


//===========================================================================
AlgObj2DInt::AlgObj2DInt(int degree)
    : degree_(degree), power_basis_(false)
//===========================================================================
{
}


//===========================================================================
AlgObj2DInt::AlgObj2DInt(const vector<Alg2DElem>& terms)
    : terms_(terms), power_basis_(true)
//===========================================================================
{
    // We must run through terms_ to find the degree_.
    int degree = 0;
    for (size_t ki = 0; ki < terms_.size(); ++ki) {
	int sum_deg = terms_[ki].degrees_[0] +
	    terms_[ki].degrees_[1];
	if (sum_deg > degree) {
	    degree = sum_deg;
	}
    }

    degree_ = degree;
}


//===========================================================================
AlgObj2DInt::~AlgObj2DInt()
//===========================================================================
{
}


//===========================================================================
Alg2DElem AlgObj2DInt::term(int index)
//===========================================================================
{
    ASSERT(index < int(terms_.size()));
    return terms_[index];
}


} // namespace Go
